/*
 *
 *
 */

#include <defs.h>

struct aimd_regusers *reguserslist = NULL;
pthread_mutex_t reguserslist_lock = PTHREAD_MUTEX_INITIALIZER;
struct aimd_clientinfo *clientlist = NULL;
pthread_mutex_t clientlist_lock = PTHREAD_MUTEX_INITIALIZER;



/* 
 * Register a new user to let them log in
 */
int aimd_addreguser(char *sn, char *password, unsigned short regstatus, char *email)
{
  struct aimd_regusers *newuser;

  newuser = (struct aimd_regusers *)malloc(sizeof(struct aimd_regusers));
  memset(newuser, 0, sizeof(struct aimd_regusers));

  strcpy(newuser->userinfo.sn, sn);
  strcpy(newuser->password, password);
  newuser->email = malloc(strlen(email)+1);
  strcpy(newuser->email, email);
  newuser->regstatus = regstatus;

  pthread_mutex_lock(&reguserslist_lock);
  newuser->next = reguserslist;
  reguserslist = newuser;
  pthread_mutex_unlock(&reguserslist_lock);

  return 0;
}

/*
 * Find a registered user by SN.
 */
struct aimd_regusers *aimd_findreguser(char *sn)
{
  struct aimd_regusers *cur;

  if (!sn)
    return NULL;
  
  pthread_mutex_lock(&reguserslist_lock);
  for (cur = reguserslist; cur; cur = cur->next) {
    if (aim_sncmp(cur->userinfo.sn, sn) == 0)
      break;
  }
  pthread_mutex_unlock(&reguserslist_lock);

  return cur;
}

/*
 * Find a connected client by SN.
 */
struct aimd_clientinfo *aimd_findclient(char *sn)
{
  struct aimd_clientinfo *cur;
  int found = 0;

  pthread_mutex_lock(&clientlist_lock);
  for(cur = clientlist; cur; cur = cur->next) {
    if (aim_sncmp(cur->sn, sn) == 0) {
      found = 1;
      break;
    }
  }
  pthread_mutex_unlock(&clientlist_lock);

  if (found) {
    return cur;
  }
  return NULL;
}

/*
 * Find a connected client by last cookie it was assigned
 */
struct aimd_clientinfo *aimd_findclient_bycookie(unsigned char *cookie)
{
  struct aimd_clientinfo *cur;

  pthread_mutex_lock(&clientlist_lock);
  for(cur = clientlist; cur; cur = cur->next) {
    if (memcmp(cur->lastcookie, cookie, AIM_COOKIELEN) == 0) 
      break;
  }
  pthread_mutex_unlock(&clientlist_lock);

  return cur;
}

/*
 * Register a new client connection
 */
struct aimd_clientinfo *aimd_addclient(char *sn, struct aim_session_t *sess) 
{
  struct aimd_clientinfo *newinfo;

  if (!sn || !sess)
    return NULL;
  
  newinfo = (struct aimd_clientinfo *)malloc(sizeof(struct aimd_clientinfo));
  memset(newinfo, 0, sizeof(struct aimd_clientinfo));

  strcpy(newinfo->sn, sn);
  newinfo->sess = sess;

  pthread_mutex_lock(&clientlist_lock);
  newinfo->next = clientlist;
  clientlist = newinfo;
  pthread_mutex_unlock(&clientlist_lock);

  return newinfo;
}

struct aimd_clientinfo *aimd_remclient(struct aim_session_t *sess)
{
  struct aimd_clientinfo *cur = NULL, *cl = NULL;

  pthread_mutex_lock(&clientlist_lock);
  if (clientlist == NULL) {
    cl = NULL;
  } else if (clientlist->next == NULL) {
    if (clientlist->sess == sess) {
      cl = clientlist;
      clientlist = NULL;
    } else
      cl = NULL;
  } else {
    for (cur = clientlist; cur; cur = cur->next) {
      if (cur->next->sess == sess) {
	cl = cur->next;
	cur->next = cur->next->next;
	break;
      }
    }
  }
  pthread_mutex_unlock(&clientlist_lock);
  
  return cl;
}
