/*
 *
 *
 */
#ifndef __DEFS_H__
#define __DEFS_H__

#define PROFILING

#include <common.h>
#include <faim/aim.h>

#define BOSIP "127.0.0.1"
#define ADSIP "127.0.0.1"
#define CHATIP "127.0.0.1"
#define CHATNAVIP "127.0.0.1"

#define LOGINERR_URL_INVALSN "http://www.pissoff.com/invalidscreenname.html"
#define LOGINERR_URL_INVALPW "http://www.pissoff.com/invalidpassword.html"
#define LOGINERR_URL_RATEFAST "http://www.pissoff.com/ratereserve.html"

/* 
 * List of registered users (users actually allowed to access 
 * the system) 
 */
struct aimd_regusers {
  struct aim_userinfo_s userinfo;
  unsigned char password[25]; /* unencrypted */ /* XXX: len right? */ 
  unsigned short regstatus; 
  char *email; 
  struct aimd_regusers *next;
};
extern struct aimd_regusers *reguserslist;
extern pthread_mutex_t reguserslist_lock;

/*
 * List of connected clients (and possibly ones who were
 * connected but have yet to be cleared)
 */
struct aimd_clientinfo {
  char sn[MAXSNLEN+1];
  struct aim_session_t *sess;
  struct aimd_regusers *user;
  char assignedbos[16];
  unsigned char lastcookie[AIM_COOKIELEN];
  unsigned short lastservrequested;
  struct aimd_clientinfo *next;
};
extern struct aimd_clientinfo *clientlist;
extern pthread_mutex_t clientlist_lock;

/*
 * Data stored in ->aux_data of all AIM session structs
 */
struct aimd_sessdata {
  pthread_mutex_t txqueuelock;
  pthread_mutex_t rxqueuelock;
  struct aimd_clientinfo *client;
};

/*
 * Provides generic locking for a target txqueue.
 *
 * This may look odd.  The main advantage to doing
 * things that look this bad is to avoid having our
 * own copy of libfaim.  This way, we can use the 
 * installed dynlib libfaim and all will still be
 * well.
 *
 */
#define TXCALLLOCKED(x...) { \
  pthread_mutex_lock(&((struct aimd_sessdata *)sess->aux_data)->txqueuelock); \
  x \
  pthread_mutex_unlock(&((struct aimd_sessdata *)sess->aux_data)->txqueuelock); \
}

#ifdef PROFILING

typedef struct {
  unsigned long long val;
  pthread_mutex_t lock;
} prof_counter_t;

#define PROF_COUNTERINC(x) { \
  pthread_mutex_lock(&(x.lock)); \
  x.val++; \
  pthread_mutex_unlock(&(x.lock)); \
}

extern prof_counter_t prof_conncount;
extern prof_counter_t prof_packrecv;
extern prof_counter_t prof_packtrans;
extern prof_counter_t prof_msgroutes;
extern prof_counter_t prof_msgroutefails;
extern time_t prof_starttime;
#else
#define PROF_COUNTERINC(x)
#endif

/* prototypes */

/* bos.c */
int aimd_handle_servicereq(struct aim_session_t *sess, struct command_rx_struct *cur);

/* clientmanagement.c */
int aimd_addreguser(char *sn, char *password, unsigned short regstatus, char *email);
struct aimd_regusers *aimd_findreguser(char *sn);
struct aimd_clientinfo *aimd_findclient(char *sn);
struct aimd_clientinfo *aimd_findclient_bycookie(unsigned char *cookie);
struct aimd_clientinfo *aimd_addclient(char *sn, struct aim_session_t *sess);
struct aimd_clientinfo *aimd_remclient(struct aim_session_t *sess);

/* connections.c */
int aimd_rxdispatch(struct aim_session_t *sess, struct aim_session_t **new);
void *handleconn(void *arg);
void aimd_handlesnac(struct aim_session_t *sess, struct command_rx_struct *cur, unsigned short family, unsigned short subtype);

/* icbm.c */
int aim_sendimtoclient(struct aim_session_t *sess, struct aim_conn_t *conn, char *destsn, u_int flags, char *msg);
int aimd_handleicbm(struct aim_session_t *sess, struct command_rx_struct *cur);

/* login.c */
int aim_parse_incomingauth(struct aim_session_t *sess, struct command_rx_struct *cur);


#endif /* __DEFS_H__ */
