/*
 *
 *
 *
 */

#include <defs.h>

#ifdef PROFILING
/* number of connections established */
prof_counter_t prof_conncount = {0, PTHREAD_MUTEX_INITIALIZER};

/* number of packets recieved */
prof_counter_t prof_packrecv = {0, PTHREAD_MUTEX_INITIALIZER};

/* number of packets transmitted */
prof_counter_t prof_packtrans = {0, PTHREAD_MUTEX_INITIALIZER};

/* number of message routes completed */
prof_counter_t prof_msgroutes = {0, PTHREAD_MUTEX_INITIALIZER};

/* number of message route failures */
prof_counter_t prof_msgroutefails = {0, PTHREAD_MUTEX_INITIALIZER};

/* start of execution */
time_t prof_starttime;

#define PROF_PRINTVAL(x) { \
  pthread_mutex_lock(&x.lock); \
  printf("%ld", x.val); \
  pthread_mutex_unlock(&x.lock); \
}

void prof_printstats(int signum)
{
  time_t elapsed;

  elapsed = time(NULL) - prof_starttime;

  printf("\n\n---- Profiling info ----\n");
  printf("Connection count: ");
  PROF_PRINTVAL(prof_conncount);
  printf("\nPacket rx count: ");
  PROF_PRINTVAL(prof_packrecv);
  printf("\nPacket tx count: ");
  PROF_PRINTVAL(prof_packtrans);
  printf("\nMessage routes completed: ");
  PROF_PRINTVAL(prof_msgroutes);
  printf("\nFailed message routes: ");
  PROF_PRINTVAL(prof_msgroutefails);
  printf("\nTime elapsed: %dsec", elapsed);
  printf("\n---- END ----\n\n");
}

#endif

/*
 * Main connection-accepting loop.
 *
 * All new connections immediatly get a thread 
 * started for them.  The lifespan of that thread
 * gets decided later when we figure out what kind
 * of data the client wants to use it for, etc.
 *
 */
int main(int argc, char **argv)
{
  int listener;
  int stayalive = 1;
  int *client; /* cant use normal variable -- its passed to non-local thread */
  pthread_t newtid;

  aimd_addreguser("diputs 81", "zooom", 42, "mid@auk.cx");
  aimd_addreguser("bleu42", "zooom", 42, "mid@auk.cx");
  aimd_addreguser("wonder", "mooz", 42, "hi@boom.com");

  if ((listener = establish(5190)) < 0) {
    perror("establish");
    exit(-1);
  }

#ifdef PROFILING 
  prof_starttime = time(NULL);
#endif

  client = NULL;
  while (stayalive) {
    if (client)
      free(client);
    client = (int *)malloc(sizeof(int));
    if ((*client = get_connection(listener)) < 0) {
      perror("get_connection");
    } else {
      PROF_COUNTERINC(prof_conncount);
      pthread_create(&newtid, NULL, handleconn, client);
      client = NULL;
    }
  }

  return 0;
}
