/* mfaim
 * Copyright (c) 1999 Josh Myer <josh@joshisanerd.com>
 * console-mode AIM client
 *
 * mfaim is based on naim;
 *  naim is Copyright (c) 1998, 1999 Daniel Reed <n@ml.org>
 * 
 * mfaim also uess libfaim,
 *  FAIM is Copyright 1998 Adam Fritzler <afritz@iname.com>
 *  FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
 */

#ifndef mfaim_h
#define mfaim_h	1

#define USE_STACK 1
#define USE_LINEEDIT 1

#include <config.h>
#include <../secs/secs.h>
#include <nfaim.h>
#include <faim/aim.h>
#include <stdarg.h>

#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif
#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef HAVE_SYS_ERRNO_H
# include <sys/errno.h>
#endif
#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_NCURSES_H
# include <ncurses.h>
#else
# ifdef HAVE_CURSES_H
#  include <curses.h>
# else
#  error Unable to locate ncurses.h; please see README.ncurses
# endif
#endif

#ifndef NO_HISTORY
# ifdef HAVE_READLINE_HISTORY_H
#  include <readline/history.h>
# else
#  ifdef HAVE_HISTORY_H
#   include <history.h>
#  else
#   error Unable to locate history.h; please see README.history
#  endif
# endif
#endif

#ifdef HAVE_ASSERT_H
# include <assert.h>
#else
# define assert(x)
#endif

#ifdef HAVE_TIME_H
# include <time.h>
# ifdef HAVE_SYS_TIME_H
#  ifdef TIME_WITH_SYS_TIME
#   include <sys/time.h>
#  endif
# endif
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# endif
#endif

// no, this isn't where you configure the colours; this just sets up the
// pair numerics, the pairs are associated with actual colours in win.c
#define HELP_HELP	10
#define HELP_TEXT	11

#define IMWIN_BGCOLOR	20
#define IMWIN_TEXT_U	21
#define IMWIN_TEXT_R	22
#define IMWIN_STATUS	23

#define INPUT_BGCOLOR	30
#define INPUT_TEXT	31
#define INPUT_TEXT_ALT	32

#define BUDDY_BGCOLOR	40
#define BUDDY_TEXT	41
#define BUDDY_WAITING	42
#define BUDDY_OFFLINE	43
#define BUDDY_NONBUDDY	44
#define BUDDY_INFO      45

#define STAT_BGCOLOR	50
#define STAT_TEXT	51

#define BACKDROP	60

#define CONIO_MAXPARMS	10


#define MAXCMDLEN MAXMSGLEN+64

#undef MAXSNLEN
#define MAXSNLEN 30
#define MAXCRLEN 30
#define MAXWLEN 30

#define WINPUT_X faimconf.winput.widthx
#define WINPUT_Y faimconf.winput.widthy
#define WINPUT_A faimconf.winput.widthx * faimconf.winput.widthy

#define WSTATUS_X faimconf.wstatus.widthx
#define WSTATUS_Y faimconf.wstatus.widthy
#define WSTATUS_A faimconf.wstatus.widthx * faimconf.wstatus.widthy

#define WBUDDY_X faimconf.wbuddy.widthx
#define WBUDDY_Y faimconf.wbuddy.width
#define WBUDDY_A faimconf.wbuddy.widthx * faimconf.wbuddy.widthy

#define WDISPLAY_X faimconf.wdisplay.widthx
#define WDISPLAY_Y faimconf.wdisplay.widthy
#define WDISPLAY_A faimconf.wdisplay.widthx * faimconf.wdisplay.widthy


typedef struct mfaimses_ts {
       char    sn[MAXSNLEN+1];
       int     lag, connected;
       time_t  online;
} mfaimses_t;

typedef struct buddywin_ts {
  char winname[MAXSNLEN+1];
  char waiting,	// text waiting to be read (overrides
                // offline and isbuddy in bupdate())
    offline,	// window currently points to offline person
    notbuddy,	// window is for a non-buddy
    keepafterso,// keep window open after buddy signs off?
    away,       // buddy is away
    chat;	// chat window (multiuser)
  WINDOW *win;
  struct buddywin_ts *next;
  struct buddywin_ts *prev;
  time_t informed;
} buddywin_t;

typedef struct buddylist_ts {
  char	screenname[MAXSNLEN+1],
    group[21],
    notes[41],
    chat;
  struct buddylist_ts	*next;
} buddylist_t;

typedef struct ignorelist_ts {
  char	screenname[MAXSNLEN+1],
    notes[128];
  struct ignorelist_ts *next;
  time_t informed;
} ignorelist_t;

typedef struct {
  char	screenname[MAXSNLEN+1],
    password[16],
    logfile[128],
    *profile,
    nolog,
    aimserver[128];
  buddylist_t	*buddies;
  ignorelist_t	*idiots;
  int	cols;
  struct {
    char	fore, back;
  } colors[64];
  struct {
    int	startx, starty,
      widthx, widthy;
  } wstatus, winput, wbuddy, wdisplay;
} faimconf_t;

typedef struct sn_queue_ts {
  char screenname[MAXSNLEN+1];
  struct sn_queue_ts *next;
  int index;
} sn_queue_t;

/* aimio.c */
void	mfaim_budprof(struct aim_conn_t *);

/* buddy.c */
void	bsetup(void);
void	bnewwin(char *);
void	wbuddy_update(void);
void    wdirty(buddywin_t *);
void    wundirty(buddywin_t *);
void	bcoming(char *);
void	bgoing(char *);
void	wremove(buddywin_t *);
buddywin_t *wgetwin(char *);
buddywin_t *wgetwin_u(char*);
buddywin_t *cgetwin(char *);
buddywin_t *cgetwin_u(char*);
buddywin_t *bgetwin(char *);
buddywin_t *bgetwin_u(char*);
void	bclearall(void);
WINDOW  *bnewwindow(void);
void    wdisplay_update(time_t);

/* conio.c */
void    conio_sendim(char *, char *);
int	conio_handlecmd(char *);
void    conio_who(char *);
void    conio_whomp(char *, char *, int, int,int);
int     conio_yank(char *, char *, int);
void    conio_print_input(char *, int, int);
void	gotkey(int);
void    conio_joinroom(char *);
void    conio_invite(char *);

/* events.c */
void	updateidletime(void);
void	event_handle(time_t);

/* hamster.c */
void	logim(char *, char *, char *);
void	mfaim_send_im(char *, char *);
void	mfaim_send_im_away(char *, char *);
void	setaway(void);
void	unsetaway(void);
void	sendaway(char *);

/* helpcmd.c */
void	help_printhelp(char *);

/* hwprintf.c */
int     mfaim_dprintf(WINDOW *, int, const char *, ...);
int	hwprintf(WINDOW *, int, const char *, ...);
unsigned long	parsehtml(WINDOW *, char *, char *, char *, char *);
/* rc.c */
void	killconf(faimconf_t *);
buddylist_t	*rgetlist(faimconf_t *, char *);
char	*rmakebuddyar(faimconf_t *);
int	mfaim_read_config(char *);
void    rjoin(void);

/* rodents.c */
int	aimcmp(char *, char *);
char    *sn_normalize(char *, char*);
char    *cr_normalize(char *, char*);
void sn_queue_enq(char *);
char * sn_queue_peek(void);
char *sn_queue_deq(char *);
void sn_queue_rm(char *);

/* script.c */
int	script_doalias(char *, char *);

/* set.c */
void	set_setvar(char *);

/* win.c */
void	wsetp(void);
void	wsetup(void);
int	wcols(faimconf_t *);
void	winitwin(WINDOW *, int);
int	wprintf(WINDOW *, int, int, const char *, ...);
void	wshutitdown(void);

#endif /* mfaim_h */
